import json
from collections import namedtuple
from json import JSONEncoder
import os
import pathlib
from pathlib import Path
import pandas as pd


from datetime import datetime
from datetime import timedelta


def offsetdt(
    time_str,
    nbhoursoffset=4,
    date_format_str="%m/%d/%Y %H:%M:%S",
    output_dt_format="%Y-%m-%d %H:%M:%S",
):
    given_time = datetime.strptime(time_str, date_format_str)
    final_time = given_time + timedelta(hours=nbhoursoffset)
    final_time_str = final_time.strftime(output_dt_format)
    return final_time_str


def fixdtindf(
    dfsrc,
    fieldname="dt",
    n=4,
    date_format_str="%m/%d/%Y %H:%M:%S",
    output_dt_format="%Y-%m-%d %H:%M:%S",
):
    # date_format_str = '%m/%d/%Y %H:%M:%S' #06/08/20.2.89:00:00
    dfo = pd.DataFrame()
    for index, row in dfsrc.iterrows():
        # print(index)
        # Given timestamp in string
        time_str = row[fieldname]
        # create datetime object from timestamp string
        given_time = datetime.strptime(time_str, date_format_str)
        # print('Given Time: ', given_time)
        # Add 2 hours to datetime object
        final_time = given_time + timedelta(hours=n)
        # print('Final Time : ', final_time)
        # Convert datetime object to string in specific format
        final_time_str = final_time.strftime(output_dt_format)  # 20.2.86-08 13:00:00
        # print('Final Time as string object: ', final_time_str)
        dfsrc.at[index, fieldname] = final_time_str
        # _df[index][fieldname]=final_time_str
        # row['dt'] = final_time_str
        # df[index]['dt'] = final_time_str
        # dfo[index]=row
    return dfsrc


# @title Functions Json decode dict
def povRequestDecoder(povReqDict):
    return jgt_povRequestDecoder(povReqDict)  # name changes


def jgt_povRequestDecoder(povReqDict):
    return namedtuple("X", povReqDict.keys())(*povReqDict.values())


def jgtjson_Decoder(dicsrc):
    return namedtuple("X", dicsrc.keys())(*dicsrc.values())


def json2prop(jsonsrc):
    _dic = json.loads(jsonsrc)
    return namedtuple("X", _dic.keys())(*_dic.values())


def d2p(dicsrc):
    return namedtuple("X", dicsrc.keys())(*dicsrc.values())


def json2dict(jsonsrc):
    _dic = json.loads(jsonsrc)
    return namedtuple("X", _dic.keys())(*_dic.values())


def jsonfile2prop(__jsonfile, quiet=False):
    _jsonfile = pathlib.Path(__jsonfile).resolve()
    # _jsonfile= os.path.abspath(__jsonfile)
    with open(_jsonfile, "r") as f:
        d = f.read()
        return json.loads(d)


def jsonfile2dict(jsoninputfile):
    _dic = jsonfile2prop(jsoninputfile)
    return namedtuple("X", _dic.keys())(*_dic.values())


def json_DecodeHierarchy(dicParentJSON, targetProp):
    dicParent = json.loads(dicParentJSON)
    dicResult = dicParent[targetProp]
    # print(_dic)
    return namedtuple("X", dicResult.keys())(*dicResult.values())


def jgtjson_DecodeHierarchy(dicParentJSON, targetProp):
    return json_DecodeHierarchy(dicParentJSON, targetProp)


def cnf_Decoder(cnfDict):
    return json.loads(cnfDict)
    # return namedtuple('X', cnfDict.keys())(*cnfDict.values())


def jgt_cnf_Decoder(cnfDict):
    return jgt_cnf_Decoder(cnfDict)



import os
import json


def read_value_from_config(var_name_to_read, pds_server_url_default):
    # Check if config.json exists in the current directory
    if os.path.exists("config.json"):
        config_path = "config.json"
    else:
        # Check if config.json exists in $HOME directory
        home_dir = os.path.expanduser("~")
        config_path = os.path.join(home_dir, "config.json")

    # Read the variable value from the config file
    if os.path.exists(config_path):
        with open(config_path) as f:
            config = json.load(f)
            variable_value = config.get(var_name_to_read, pds_server_url_default)
    else:
        variable_value = pds_server_url_default

    return variable_value



import threading
from typing import Callable

from . import fxcorepy


class TableManagerListener(fxcorepy.AO2GTableManagerListener):
    """The class implements the abstract class AO2GTableManagerListener and calls the passed function when the table manager status changes."""
    def __init__(self, on_status_changed_callback: Callable[
        [fxcorepy.AO2GTableManagerListener, fxcorepy.O2GTableManager, fxcorepy.O2GTableManagerStatus], None] = None) \
            -> None:
        """ The constructor.
        
            Parameters
            ----------
            on_status_changed_callback : typing.Callable[[AO2GTableManagerListener, O2GTableManager, O2GTableManagerStatus], None]
                The function that is called when the table manager status changes.
        
            Returns
            -------
            None
        
        """
        fxcorepy.AO2GTableManagerListener.__init__(self)
        self._on_status_changed_callback = on_status_changed_callback
        self._status = fxcorepy.O2GTableManagerStatus.TABLES_LOADING
        self._semaphore = threading.Event()

    @property
    def status(self) -> fxcorepy.O2GTableManagerStatus:
        """ Gets the current status of the table manager.
        
            Returns
            -------
            O2GTableManagerStatus
        
        """
        return self._status

    def set_callback(self, on_status_changed_callback: Callable[
        [fxcorepy.AO2GTableManagerListener, fxcorepy.O2GTableManager, fxcorepy.O2GTableManagerStatus], None]) \
            -> None:
        """ Sets a callback function.
        
            Parameters
            ----------
            on_status_changed_callback : 
                The function that is called when the table manager status changes.
        
            Returns
            -------
            None
        
        """
        self._on_status_changed_callback = on_status_changed_callback

    def on_status_changed(self, status: fxcorepy.O2GTableManagerStatus, table_manager: fxcorepy.O2GTableManager) \
            -> None:  # native call
        """ Implements the method AO2GTableManagerListener.on_status_changed and calls the function that processes notifications about the table manager status change. The function is passed in the constructor or set by the set_callback method.
        
            Returns
            -------
            
        
        """
        self._status = status
        if self._on_status_changed_callback:
            self._on_status_changed_callback(self, table_manager, status)

        if status != fxcorepy.O2GTableManagerStatus.TABLES_LOADING:
            self._semaphore.set()

    def wait_event(self) -> bool:
        """Reserved for future use."""
        return self._semaphore.wait(30)

    def reset(self) -> None:
        """ Resets the flag that is set when the table manager status changes to TABLES_LOADED or TABLES_LOAD_FAILED.
        
            Returns
            -------
            None
        
        """
        self._status = fxcorepy.O2GTableManagerStatus.TABLES_LOADING
        self._semaphore.clear()




# %%
#@title Import
import cv2
import numpy as np
import io
from matplotlib import pyplot as plt


# %% [markdown]
# # Brightener

# %% [markdown]
# ## Automated

# %%
def img_histogram_view(hist,gray,minimum_gray,maximum_gray):
    new_hist = cv2.calcHist([gray],[0],None,[256],[minimum_gray,maximum_gray])
    plt.plot(hist)
    plt.plot(new_hist)
    plt.xlim([0,256])
    plt.show()

    # Automatic brightness and contrast optimization with optional histogram clipping
def automatic_brightness_and_contrast(image, clip_hist_percent=11,showhistogram=False):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # Calculate grayscale histogram
    hist = cv2.calcHist([gray],[0],None,[256],[0,256])
    hist_size = len(hist)

    # Calculate cumulative distribution from the histogram
    accumulator = []
    accumulator.append(float(hist[0]))
    for index in range(1, hist_size):
        accumulator.append(accumulator[index -1] + float(hist[index]))

    # Locate points to clip
    maximum = accumulator[-1]
    clip_hist_percent *= (maximum/100.0)
    clip_hist_percent /= 2.0

    # Locate left cut
    minimum_gray = 0
    while accumulator[minimum_gray] < clip_hist_percent:
        minimum_gray += 1

    # Locate right cut
    maximum_gray = hist_size -1
    while accumulator[maximum_gray] >= (maximum - clip_hist_percent):
        maximum_gray -= 1

    # Calculate alpha and beta values
    alpha = 255 / (maximum_gray - minimum_gray)
    beta = -minimum_gray * alpha

    '''
    # Calculate new histogram with desired range and show histogram 
    new_hist = cv2.calcHist([gray],[0],None,[256],[minimum_gray,maximum_gray])
    plt.plot(hist)
    plt.plot(new_hist)
    plt.xlim([0,256])
    plt.show()
    '''
    if showhistogram:
        img_histogram_view(hist,gray,minimum_gray,maximum_gray)


    auto_result = cv2.convertScaleAbs(image, alpha=alpha, beta=beta)
    # if showhistogram:
    #     img_histogram_view(hist,gray,minimum_gray,maximum_gray)

    return (auto_result, alpha, beta)


# %%

def abc(imgctx,resultprefix,seqit=25,ext='.png',showhistogram=False):
  resultfn=resultprefix+'-'+str(seqit)+ext
  image = cv2.imread(imgctx)
  auto_result, alpha, beta = automatic_brightness_and_contrast(image,seqit,showhistogram)
  print('<!--alpha', alpha,'-->')
  print('<!--beta', beta,'-->')
  #cv2.imshow('auto_result', auto_result)
  cv2.imwrite(resultfn, auto_result)
  print('#### ' + str(seqit))
  print('!['+str(seqit)+']('+resultfn+')')
  return auto_result



from typing import Callable, Dict, Any
import atexit
import threading
import logging
from datetime import datetime
import warnings
import os
import platform

with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=RuntimeWarning, module="importlib._bootstrap")
    # your code here


import numpy as np
try:
    from . import fxcorepy as fxcorepy
except:
    pass
from .TableManagerListener import TableManagerListener
from .SessionStatusListener import SessionStatusListener
from .errors import (RequestFailedError,
                                 TableManagerError,
                                 LoginFailedError,
                                 TimeFrameError)
from .ResponseListener import ResponseListener, ResponseListenerAsync


@atexit.register
def on_exit():
    fxcorepy.O2GTransport.finalize_wrapper()


class ForexConnect:
    """The class is intended for working with a session."""
    _SESSION = None
    _STATUS_LISTENER = None
    _TABLE_MANAGER_LISTENER = None
    _START_URL = "https://fxcorporate.com/Hosts.jsp"
    _LISTENER = None

    OFFERS = fxcorepy.O2GTableType.OFFERS
    ACCOUNTS = fxcorepy.O2GTableType.ACCOUNTS
    TRADES = fxcorepy.O2GTableType.TRADES
    CLOSED_TRADES = fxcorepy.O2GTableType.CLOSED_TRADES
    ORDERS = fxcorepy.O2GTableType.ORDERS
    MESSAGES = fxcorepy.O2GTableType.MESSAGES
    SUMMARY = fxcorepy.O2GTableType.SUMMARY

    class TableUpdateType(fxcorepy.O2GTableUpdateType):
        pass

    class ResponseType(fxcorepy.O2GResponseType):
        pass

    class TableManagerStatus(fxcorepy.O2GTableManagerStatus):
        pass

    def __init__(self) -> None:
        self._session = ForexConnect._SESSION
        self._status_listener = ForexConnect._STATUS_LISTENER
        self._table_manager_listener = ForexConnect._TABLE_MANAGER_LISTENER
        self._prev_response_listener = None
        self._async_response_listener = ResponseListenerAsync(self)
        self._async_response_listener_subscribed = False
        self._start_url = ForexConnect._START_URL
        self._session_id = None
        self._pin = None
        self._table_manager = None
        self._com = None

    @property
    def session(self) -> fxcorepy.O2GSession:
        """ Gets an instance of the current session.
        
            Returns
            -------
            O2GSession
        
        """
        return self._session

    @property
    def table_manager(self) -> fxcorepy.O2GTableManager:
        """ Gets the current table manager of the session.
        
            Returns
            -------
            O2GTableManager
        
        """
        return self._table_manager

    @property
    def response_listener(self) -> ResponseListenerAsync:
        """Reserved for future use."""
        return self._async_response_listener

    @property
    def login_rules(self) -> fxcorepy.O2GLoginRules:
        """ Gets the rules used for the currently established session.
        
            Returns
            -------
            O2GLoginRules
        
        """
        login_rules = self._session.login_rules

        if login_rules is None:
            raise Exception("Cannot get login rules")

        return login_rules

    def __enter__(self):
        return self

    def __exit__(self, tp, value, traceback):
        self.logout()

    def _login(self,
               login_function,
               login_params,
               session_id: str = None,
               pin: str = None,
               session_status_callback: Callable[[fxcorepy.O2GSession,
                                                  fxcorepy.AO2GSessionStatus.O2GSessionStatus], None] = None,
               use_table_manager: bool = True,
               table_manager_callback: Callable[[fxcorepy.O2GTableStatus,
                                                 fxcorepy.O2GTableManager],
                                                None] = None) -> fxcorepy.O2GSession:

        self._session_id = session_id
        self._pin = pin
        self._session = fxcorepy.O2GTransport.create_session()

        if use_table_manager:
            self._table_manager_listener = TableManagerListener(table_manager_callback)
            self._session.use_table_manager(fxcorepy.O2GTableManagerMode.YES,
                                            self._table_manager_listener)

        self._status_listener = SessionStatusListener(self._session,
                                                      session_id,
                                                      pin,
                                                      session_status_callback)
        self._session.subscribe_session_status(self._status_listener)
        self._status_listener.reset()

        login_function = getattr(self._session, login_function)
        if not login_function(*login_params):
            raise LoginFailedError("The login method returned an exception."
                                   " This may be caused by the incorrect session status.")

        timeout = self._status_listener.wait_event()
        status_connect = self._status_listener.connected

        self._table_manager = self._session.table_manager

        if not timeout:
            raise LoginFailedError("Wait timeout exceeded")

        if not status_connect:
            if self._status_listener.last_error:
                raise LoginFailedError(self._status_listener.last_error)
            else:
                raise LoginFailedError("Not connected")

        if use_table_manager:
            self._table_manager_listener.wait_event()
            if self._table_manager.status != fxcorepy.O2GTableManagerStatus.TABLES_LOADED:
                raise LoginFailedError("Table manager not ready")

        return self._session

    def login(self,
              user_id: str,
              password: str,
              url: str = _START_URL,
              connection: str = "Demo",
              session_id: str = None,
              pin: str = None,
              session_status_callback: Callable[[fxcorepy.O2GSession,
                                                 fxcorepy.AO2GSessionStatus.O2GSessionStatus], None] = None,
              use_table_manager: bool = True,
              table_manager_callback: Callable[[fxcorepy.O2GTableStatus,
                                                fxcorepy.O2GTableManager],
                                               None] = None) -> fxcorepy.O2GSession:
        """ Creates a trading session and starts the connection with the specified trade server.
        
            Parameters
            ----------
            user_id : str
                The user name.
            password : str
                The password.
            url : str
                The URL of the server. The URL must be a full URL, including the path to the host descriptor.
            connection : str
                The name of the connection, for example Demo or Real.
            session_id : str
                The database name. Required only for users who have accounts in more than one database.
            pin : str
                The PIN code. Required only for users who have PIN codes.
            session_status_callback : typing.Callable[[O2GSession, AO2GSessionStatus.O2GSessionStatus], None]
                The function that is called when the session status changes.
            use_table_manager : bool
                Defines whether ForexConnect is started with the table manager.
            table_manager_callback : typing.Callable[[O2GTableManager, O2GTableStatus], None]
                The function that is called when the table manager status changes.
        
            Returns
            -------
            O2GSession
        
        """
        return self._login("login", (user_id, password, url, connection), session_id, pin, session_status_callback,
                           use_table_manager, table_manager_callback)

    def login_with_token(self,
                         user_id: str,
                         token: str,
                         url: str = _START_URL,
                         connection: str = "Demo",
                         session_id: str = None,
                         pin: str = None,
                         session_status_callback: Callable[[fxcorepy.O2GSession,
                                                            fxcorepy.AO2GSessionStatus.O2GSessionStatus], None] = None,
                         use_table_manager: bool = True,
                         table_manager_callback: Callable[[fxcorepy.O2GTableStatus,
                                                           fxcorepy.O2GTableManager],
                                                          None] = None) -> fxcorepy.O2GSession:
        """ Creates a second trading session and starts the connection with the specified trade server using a token.
        
            Parameters
            ----------
            user_id : str
                The user name.
            token : str
                The token.
            url : str
                The URL of the server. The URL must be a full URL, including the path to the host descriptor.
            connection : str
                The name of the connection, for example Demo or Real.
            session_id : str
                The database name. Required only for users who have accounts in more than one database.
            pin : str
                The PIN code. Required only for users who have PIN codes. The default value of the parameter is None.
            session_status_callback : typing.Callable[[O2GSession, AO2GSessionStatus.O2GSessionStatus], None]
                The function that is called when the session status changes. The default value of the parameter is None.
            use_table_manager : bool
                Defines whether ForexConnect is started with the table manager. The default value of the parameter is True.
            table_manager_callback : typing.Callable[[O2GTableStatus, O2GTableManager], None]
                The function that is called when the table manager status changes. The default value of the parameter is None
        
            Returns
            -------
            O2GSession
        
        """
        return self._login("login_with_token", (user_id, token, url, connection), session_id, pin,
                           session_status_callback, use_table_manager, table_manager_callback)

    def logout(self) -> None:
        self.unsubscribe_response()
        if self._session is None or \
                self._session.session_status == fxcorepy.AO2GSessionStatus.O2GSessionStatus.DISCONNECTED:
            return
        """ Closes the trading session and connection with the trade server.
        
            Returns
            -------
            None
        
        """
        self._status_listener.reset()
        self._session.logout()
        self._status_listener.wait_event()
        self._session.unsubscribe_session_status(self._status_listener)

        self._session = None
        self._status_listener = None
        self._table_manager = None

    def set_session_status_listener(self, listener: Callable[
        [fxcorepy.O2GSession, fxcorepy.AO2GSessionStatus.O2GSessionStatus], None]) \
            -> None:
        """ Sets a session status listener on login or sets a new session status listener when necessary.
        
            Parameters
            ----------
            listener : typing.Callable[[O2GSession, AO2GSessionStatus.O2GSessionStatus], None]
                The function that is called when the session status changes.
        
            Returns
            -------
            None
        
        """
        self._status_listener.set_callback(listener)

    def get_table_reader(self, table_type: fxcorepy.O2GTable,
                         response: fxcorepy.O2GResponse = None) -> fxcorepy.O2GGenericTableResponseReader:
        """ Gets an instance of a table reader.
        
            Parameters
            ----------
            table_type : O2GTableType
                The identifier of the table (see O2GTableType).
            response : O2GResponse
                An instance of O2GResponse to get a reader for.
        
            Returns
            -------
            typing.object
        
        """
        if response is None:
            login_rules = self._session.login_rules
            response = login_rules.get_table_refresh_response(table_type)

        return self.create_reader(response)

    def get_table(self, table_type: fxcorepy.O2GTable) -> fxcorepy.O2GTable:
        """ Gets a specified table.
        
            Parameters
            ----------
            table_type : O2GTableType
                The identifier of the table. For a complete list of tables, see O2GTableType.
        
            Returns
            -------
            O2GTable
        
        """
        if self._table_manager_listener is None:
            raise TableManagerError(
                'Need login with flag "useTableManager"')

        if not self._table_manager_listener.wait_event():
            raise TableManagerError("Wait timeout exceeded")

        if self._table_manager_listener.status == fxcorepy.O2GTableManagerStatus.TABLES_LOAD_FAILED:
            raise TableManagerError(
                "Cannot refresh all tables of table manager")

        return self._session.table_manager.get_table(table_type)

    def subscribe_response(self) -> None:
        """Reserved for future use."""
        if self._async_response_listener_subscribed:
            return
        self._async_response_listener_subscribed = True
        self._session.subscribe_response(self._async_response_listener)

    def unsubscribe_response(self) -> None:
        """Reserved for future use."""
        if not self._async_response_listener_subscribed:
            return
        self._async_response_listener_subscribed = False
        self._session.unsubscribe_response(self._async_response_listener)

    def send_request_async(self,
                           request: fxcorepy.O2GRequest,
                           listener: ResponseListener = None):
        """ Sends a request.
        
            Parameters
            ----------
            request : O2GRequest
                An instance of O2GRequest.
            listener : ResponseListener
                An instance of ResponseListener. The default value of the parameter is None.
        
            Returns
            -------
            None
        
        """
        if listener is None:
            listener = ResponseListener(self._session)
        self.subscribe_response()
        request_ids = []
        Common.fill_request_ids(request_ids, request)
        listener.set_request_ids(request_ids)
        self._async_response_listener.add_response_listener(listener)
        self._session.send_request(request)

    def send_request(self,
                     request: fxcorepy.O2GRequest,
                     listener: ResponseListener = None) -> Any:
        """ Sends a request and returns the appropriate response reader or a bool value.
        
            Parameters
            ----------
            request : O2GRequest
                An instance of O2GRequest.
            listener : ResponseListener
                An instance of ResponseListener. The default value of the parameter is None.
        
            Returns
            -------
            typing.Any
        
        """
        if threading.current_thread() != threading.main_thread():
            logging.warning("Calling the send_request method is not from the main thread. "
                            "If you call the send_request method from a callback, you can get the application freezed. "
                            "It is recommended to use send_request_async method.")
        if listener is None:
            listener = ResponseListener(self._session)
        listener.reset()
        self.send_request_async(request, listener)

        if not listener.wait_event():
            self._async_response_listener.remove_response_listener(listener)
            raise RequestFailedError("Wait timeout exceeded")

        self._async_response_listener.remove_response_listener(listener)

        error = listener.error

        if error is None:
            return self.create_reader(listener.response)

        raise RequestFailedError(error)

    def get_history(self,
                    instrument: str,
                    timeframe: str,
                    date_from: datetime = None,
                    date_to: datetime = None,
                    quotes_count: int = -1,
                    candle_open_price_mode=fxcorepy.O2GCandleOpenPriceMode.PREVIOUS_CLOSE
                    ) -> np.ndarray:
        """ Gets price history of a certain instrument with a certain timeframe for a specified period or a certain number of bars/ticks.
        
            Parameters
            ----------
            instrument : str
                The symbol of the instrument. The instrument must be one of the instruments the ForexConnect session is subscribed to.
            timeframe : str
                The unique identifier of the timeframe. For details, see What is Timeframe?.
            date_from : datetime.datetime
                The date/time of the oldest bar/tick in the history.
            date_to : datetime.datetime
                The date/time of the most recent bar/tick in the history.
            quotes_count : int
                The number of bars/ticks in the history. The parameter is optional.
            candle_open_price_mode : 
                The candles open price mode that indicates how the open price is determined.
        
            Returns
            -------
            numpy.ndarray
        
        """
        


        if self._com is None:
            var_jgt_dir = "/var/lib/jgt"
           
            if platform.system() == 'Windows':
                var_jgt_dir = "C:\\var\\lib\\jgt"

            
            if os.path.isdir(var_jgt_dir) and os.access(var_jgt_dir, os.W_OK):
                history_dir = os.path.join(var_jgt_dir,"History")
            else:
                #try to create in $HOME/.cache/jgt/History
                history_dir = os.path.join(os.path.expanduser("~"), ".cache", "jgt", "History")
                try:
                    os.makedirs(history_dir, exist_ok=True)
                except:
                    history_dir = os.path.join(os.getcwd(), "History")
                    print("using ./History, might want to create :  sudo mkdir -p -m 777 " + var_jgt_dir)
                
            self._com = fxcorepy.PriceHistoryCommunicatorFactory.create_communicator(
                        self._session, history_dir)

        timeframe = self._com.timeframe_factory.create(timeframe)
        if not timeframe:
            raise TimeFrameError("Timeframe is incorrect")

        while not self._com.is_ready:
            pass

        self._com.candle_open_price_mode = candle_open_price_mode
        #if date_from is not None:
            #print("We are in ForexConnect: " + str(date_from) + ", to:" + str(date_to))
        reader = self._com.get_history(instrument, timeframe, date_from, date_to, quotes_count)
        if timeframe.unit == fxcorepy.O2GTimeFrameUnit.TICK:
            result = np.zeros(len(reader), np.dtype([('Date', "M8[ns]"), ('Bid', "f8"), ("Ask", "f8")]))
            idx = -1
            for row in reader:
                idx += 1
                result[idx]['Date'] = np.datetime64(row.date)
                result[idx]['Bid'] = row.bid
                result[idx]['Ask'] = row.ask
            return result
        else:
            result = np.zeros(len(reader), np.dtype([
                ('Date', "M8[ns]"),
                ('BidOpen', 'f8'), ('BidHigh', 'f8'), ('BidLow', 'f8'), ('BidClose', 'f8'),
                ('AskOpen', 'f8'), ('AskHigh', 'f8'), ('AskLow', 'f8'), ('AskClose', 'f8'),
                ('Volume', 'i4')]))
            idx = -1
            for row in reader:
                idx += 1
                result[idx]['Date'] = np.datetime64(row.date)
                result[idx]['BidOpen'] = row.bid_open
                result[idx]['BidHigh'] = row.bid_high
                result[idx]['BidLow'] = row.bid_low
                result[idx]['BidClose'] = row.bid_close
                result[idx]['AskOpen'] = row.ask_open
                result[idx]['AskHigh'] = row.ask_high
                result[idx]['AskLow'] = row.ask_low
                result[idx]['AskClose'] = row.ask_close
                result[idx]['Volume'] = row.volume
            return result

    def create_reader(self,
                      response: fxcorepy.O2GResponse) -> Any:
        """ Creates a reader for a certain response using the method O2GResponseReaderFactory.create_reader.
        
            Parameters
            ----------
            response : O2GResponse
                An instance of O2GResponse.
        
            Returns
            -------
            typing.Any
        
        """
        factory = self._session.response_reader_factory
        if factory is None:
            raise Exception("Create ResponseReaderFactory failed")
        return factory.create_reader(response)

    def create_request(self, params: Dict[fxcorepy.O2GRequestParamsEnum, str],
                       request_factory: fxcorepy.O2GRequestFactory = None,
                       root: bool = True) -> fxcorepy.O2GRequest:
        """ Creates a request.
        
            Parameters
            ----------
            params : typing.Dict[O2GRequestParamsEnum, str]
                The request parameters. See O2GRequestParamsEnum.
            request_factory : O2GRequestFactory
                An instance of O2GRequestFactory.
            root : bool
                Defines whether the request is root. The default value of the parameter is True.
        
            Returns
            -------
            O2GRequest
        
        """
        if request_factory is None:
            request_factory = self._session.request_factory

            if request_factory is None:
                raise Exception("Can not create request factory")

        value_map = request_factory.create_value_map()
        items = params.items()

        for k, v in items:
            if isinstance(v, dict):
                value_map.append_child(
                    self.create_request(v, request_factory, False))

            else:
                if isinstance(v, str):
                    value_map.set_string(k, v)

                if isinstance(v, int):
                    value_map.set_int(k, v)

                if isinstance(v, bool):
                    value_map.set_bool(k, v)

                if isinstance(v, float):
                    value_map.set_double(k, v)

        if root:
            request = request_factory.create_order_request(value_map)
            if request is None:
                raise Exception(request_factory.last_error)
            return request

        else:
            return value_map

    def create_order_request(self, order_type: str,
                             command: fxcorepy.Constants.Commands = fxcorepy.Constants.Commands.CREATE_ORDER,
                             **kwargs: str) -> fxcorepy.O2GRequest:
        """ Creates a request for creating an order of a specified type using specified parameters.
        
            Parameters
            ----------
            order_type : str
                The type of the order. See Contansts.Orders.
            command : Commands
                The command. The default value of the parameter is fxcorepy.Constants.Commands.CreateOrder.
            str : kwargs
                The order parameters. For a full list of possible request parameters, see O2GRequestParamsEnum.
        
            Returns
            -------
            O2GRequest
        
        """
        if command is None:
            command = fxcorepy.Constants.Commands.CREATE_ORDER

        params = {
            fxcorepy.O2GRequestParamsEnum.COMMAND: command,
            fxcorepy.O2GRequestParamsEnum.ORDER_TYPE: order_type
        }

        for param in kwargs:
            enum = fxcorepy.O2GRequestParamsEnum.names[param]
            params[enum] = kwargs[param]

        return self.create_request(params)

    def get_timeframe(self, str_timeframe: str) -> fxcorepy.O2GTimeFrame:
        """ Gets an instance of a timeframe.
        
            Parameters
            ----------
            str_timeframe : str
                The unique identifier of the timeframe.
        
            Returns
            -------
            O2GTimeframe
        
        """
        return self._session.request_factory.timeframe_collection.get_by_id(
            str_timeframe)

    @staticmethod
    def parse_timeframe(timeframe: str) -> tuple:
        """ Parses a timeframe into O2GTimeframeUnit and the number of units.
        
            Parameters
            ----------
            timeframe : str
                The unique identifier of the timeframe.
        
            Returns
            -------
            O2GTimeframeUnit, int
        
        """
        if (len(timeframe) < 2):
            raise TimeFrameError("Timeframe is incorrect")

        available_units = {
            't': fxcorepy.O2GTimeFrameUnit.TICK,
            'm': fxcorepy.O2GTimeFrameUnit.MIN,
            'H': fxcorepy.O2GTimeFrameUnit.HOUR,
            'D': fxcorepy.O2GTimeFrameUnit.DAY,
            'W': fxcorepy.O2GTimeFrameUnit.WEEK,
            'M': fxcorepy.O2GTimeFrameUnit.MONTH,
        }

        try:
            unit = available_units[timeframe[0]]
        except KeyError as e:
            raise TimeFrameError("Unit is incorrect")

        try:
            size = int(timeframe[1:])
        except ValueError:
            raise TimeFrameError("Size must be a number")

        return unit, size

from .common import Common



import subprocess
import os
import sys

def main():
    system_settings_path = os.path.join('/etc', 'jgt', 'settings.json')
    home_dir = os.path.expanduser("~")
    home_config_path = os.path.join(home_dir, ".jgt", "config.json")
    home_settings_path = os.path.join(home_dir, ".jgt", "settings.json")
    data_path = os.path.join(home_dir, ".jgt", "nds")
    dk_data_path = "/nds"
    full_data_path = os.path.join(home_dir, ".jgt", "fds")
    dk_full_data_path = "/fds"
    #mkdir local path
    os.makedirs(data_path, exist_ok=True)
    os.makedirs(full_data_path, exist_ok=True)
    current_directory=os.getcwd()
    
    docker_command = [
        "docker", "run"]
    
    if "--bash" in sys.argv:
        docker_command.append("-it")
    
    cli_to_run = "jgtfxcli"
    #we want to specify alternatively the cli to run and pass its arguments when running the docker container
    possible_clis=["fxtr","fxrmorder","fxaddorder","fxreport"]
    for cli in possible_clis:
        if cli in sys.argv:
            cli_to_run=cli
            #remove it from the arguments list
            sys.argv.remove(cli)
            break

    docker_image_repo_tag = "jgwill/jgt:fxcon"
    dk_home_path="/home/user"
    
    # "-v", f"{home_settings_path}:/etc/jgt/settings.json",
    docker_command = docker_command+       [ "--rm",
        "-v", f"{home_config_path}:{dk_home_path}/.jgt/config.json",
        "-v", f"{home_settings_path}:{dk_home_path}/.jgt/settings.json",
        "-v", f"{system_settings_path}:/etc/jgt/settings.json",
        "-v", f"{data_path}:{dk_data_path}",
        "-e", f"JGTPY_DATA={dk_data_path}",
        "-v", f"{full_data_path}:{dk_full_data_path}",
        "-e", f"JGTPY_DATA_FULL={dk_full_data_path}",
        "-v",f"{current_directory}:/work",
        docker_image_repo_tag]

    # Check if --bash is present in the arguments
    if "--bash" in sys.argv:
        docker_command.append("bash")
        # Remove --bash from the arguments list
        sys.argv.remove("--bash")
    else:
        docker_command.append(cli_to_run)
    print(f"Running Docker command: {' '.join(docker_command)}")
    # Append all remaining arguments passed to fxcli to the docker command
    docker_command.extend(sys.argv[1:])

    # Run the Docker command
    subprocess.run(docker_command)

if __name__ == "__main__":
    main()